@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 """Allow lazy loading of `cylc.cfgspec.globalcfg`."""
 
 
-def glbl_cfg():
+def glbl_cfg(cached=True):
     """Load and return the global configuration singleton instance."""
     from cylc.cfgspec.globalcfg import GlobalConfig
-    return GlobalConfig.get_inst()
+    return GlobalConfig.get_inst(cached=cached)
