@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,15 @@ from cylc.cycling.loader import get_point, get_point_relative
 from cylc.task_id import TaskID
 
 
+GHOST_TRANSP_HEX = "88"
+
+
+def gtk_rgb_to_hex(rgb):
+    """Return a standard 6-char hex color code from a GTK RGB color."""
+    return '#' + ''.join('%02x' % (x * 255) for x in (
+        rgb.red_float, rgb.green_float, rgb.blue_float))
+
+
 class CGraphPlain(pygraphviz.AGraph):
     """Directed Acyclic Graph class for cylc dependency graphs."""
 
@@ -50,12 +59,14 @@ class CGraphPlain(pygraphviz.AGraph):
             if node_string.startswith("__remove_"):
                 node.attr['style'] = 'dashed'
                 node.attr['label'] = u'\u2702'
+                node.attr['shape'] = 'diamond'
                 return
             raise
         label = name
         if name in self.suite_polling_tasks:
             label += "\\n" + self.suite_polling_tasks[name][3]
-        label += "\\n" + point_string
+        if not label.startswith('@'):
+            label += "\\n" + point_string
         node.attr['label'] = label
         node.attr['URL'] = node_string
 
@@ -198,7 +209,7 @@ class CGraphPlain(pygraphviz.AGraph):
                 pygraphviz.AGraph.add_node(self, left)
                 self.style_node(left)
             else:
-                attrs = {'penwidth': 2}
+                attrs = {}
                 if skipped:
                     attrs.update({'style': 'dotted', 'arrowhead': 'oinv'})
                 elif conditional and suicide:
@@ -208,13 +219,13 @@ class CGraphPlain(pygraphviz.AGraph):
                 elif suicide:
                     attrs.update({'style': 'dashed', 'arrowhead': 'dot'})
                 else:
-                    attrs.update({'style': 'solid', 'arrowhead': 'normal'})
+                    attrs.update({'arrowhead': 'normal'})
                 pygraphviz.AGraph.add_edge(self, left, right, **attrs)
                 self.style_node(left)
                 self.style_node(right)
                 self.style_edge(left, right)
 
-    def add_cycle_point_subgraphs(self, edges):
+    def add_cycle_point_subgraphs(self, edges, fgcolor):
         """Draw nodes within cycle point groups (subgraphs)."""
         point_string_id_map = {}
         for edge_entry in edges:
@@ -231,7 +242,8 @@ class CGraphPlain(pygraphviz.AGraph):
         for point_string, ids in point_string_id_map.items():
             self.add_subgraph(
                 nbunch=ids, name="cluster_" + point_string,
-                label=point_string, fontsize=28, rank="max", style="dashed"
+                label=point_string, fontsize=28, rank="max", style="dashed",
+                color=fgcolor
             )
 
     def add_subgraph(self, nbunch=None, name=None, **attr):
@@ -258,18 +270,50 @@ class CGraphPlain(pygraphviz.AGraph):
 
         return subgraph
 
+    def set_def_style(self, fgcolor, bgcolor, def_node_attr=None):
+        """Set default graph styles.
+
+        Depending on light/dark desktop color theme.
+        """
+
+        if def_node_attr is None:
+            def_node_attr = {}
+
+        # Transparent graph bg - let the desktop theme bg shine through.
+        self.graph_attr['bgcolor'] = '#ffffff00'
+
+        # graph and cluster:
+        self.graph_attr['color'] = fgcolor
+        self.graph_attr['fontcolor'] = fgcolor
+        # node outlines (or node fill if fillcolor is not defined):
+        self.node_attr['color'] = fgcolor
+        # edges:
+        self.edge_attr['color'] = fgcolor  # edges
+        # node labels:
+        if def_node_attr.get('style', '') == "filled":
+            self.node_attr['fontcolor'] = bgcolor  # node labels
+        else:
+            self.node_attr['fontcolor'] = fgcolor  # node labels
+
 
 class CGraph(CGraphPlain):
     """Directed Acyclic Graph class for cylc dependency graphs.
-    This class automatically adds node and edge attributes
-    according to the suite.rc file visualization config."""
+
+    For "cylc graph" - add node and edge attributes according to the
+    suite.rc file visualization config.
+    """
 
     def __init__(self, title, suite_polling_tasks=None, vizconfig=None):
 
         # suite.rc visualization config section
         CGraphPlain.__init__(self, title, suite_polling_tasks)
         if vizconfig is None:
-            vizconfig = {}
+            vizconfig = {
+                'default node attributes': [],
+                'default edge attributes': [],
+                'node attributes': {},
+                'edge attributes': {},
+            }
         self.vizconfig = vizconfig
 
         # graph attributes
@@ -318,10 +362,14 @@ class CGraph(CGraphPlain):
     def style_node(self, node_string):
         CGraphPlain.style_node(self, node_string)
         node = self.get_node(node_string)
-        node.attr['shape'] = 'ellipse'  # Default shape.
+        attrs = {}
         for item in self.node_attr_by_taskname(node_string):
             attr, value = [val.strip() for val in item.split('=', 1)]
+            attrs[attr] = value
             node.attr[attr] = value
+        if node.attr['style'] != 'filled' and (
+                'color' in attrs and 'fontcolor' not in attrs):
+            node.attr['fontcolor'] = node.attr['color']
         if self.vizconfig['use node color for labels']:
             node.attr['fontcolor'] = node.attr['color']
         node.attr['penwidth'] = self.vizconfig['node penwidth']
@@ -341,7 +389,8 @@ class CGraph(CGraphPlain):
     def get_graph(
             cls, suiterc, group_nodes=None, ungroup_nodes=None,
             ungroup_recursive=False, group_all=False, ungroup_all=False,
-            ignore_suicide=False, subgraphs_on=False):
+            ignore_suicide=False, subgraphs_on=False, bgcolor=None,
+            fgcolor=None):
         """Return dependency graph."""
         # Use visualization settings.
         start_point_string = (
@@ -372,11 +421,14 @@ class CGraph(CGraphPlain):
             suiterc.suite,
             suiterc.suite_polling_tasks,
             suiterc.cfg['visualization'])
+
+        graph.set_def_style(fgcolor, bgcolor, graph.node_attr)
+
         gr_edges = suiterc.get_graph_raw(
             start_point_string, stop_point_string,
             group_nodes, ungroup_nodes, ungroup_recursive,
             group_all, ungroup_all)
         graph.add_edges(gr_edges, ignore_suicide)
         if subgraphs_on:
-            graph.add_cycle_point_subgraphs(gr_edges)
+            graph.add_cycle_point_subgraphs(gr_edges, fgcolor)
         return graph
