@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@ import itertools
 import threading
 from time import time, sleep
 
+
 from cylc.gui.dot_maker import DotMaker
 from cylc.gui.util import get_id_summary
 from cylc.task_id import TaskID
@@ -252,7 +253,7 @@ class TreeUpdater(threading.Thread):
                         tetc_unix = tstart + meant
                         tnow = time()
                         if tstart > tnow:
-                            # Reportably possible via interraction with
+                            # Reportably possible via interaction with
                             # cylc reset.
                             t_info['progress'] = 0
                         elif tnow > tetc_unix:
@@ -438,7 +439,7 @@ class TreeUpdater(threading.Thread):
                     f_iter = p_iter
                     f_path = p_path
                     fam = point_string
-                    for i, fam in enumerate(named_path[:-1]):
+                    for j, fam in enumerate(named_path[:-1]):
                         # Construct family nesting for this task.
                         if fam in family_iters:
                             # Family already in tree
@@ -450,8 +451,8 @@ class TreeUpdater(threading.Thread):
                                 f_data = new_fam_data[point_string][fam]
                             except KeyError:
                                 f_data = [None] * 7
-                            if i > 0:
-                                parent_fam = named_path[i - 1]
+                            if j > 0:
+                                parent_fam = named_path[j - 1]
                             else:
                                 # point_string is the implicit parent here.
                                 parent_fam = point_string
