@@ -1,7 +1,7 @@
 #!/bin/sh
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -98,7 +98,8 @@ cylc__job__main() {
     # Otherwise, a zero padded number will be interpreted as an octal.
     export CYLC_TASK_SUBMIT_NUMBER=$((10#$(cut -d '/' -f 3 <<<"${CYLC_TASK_JOB}")))
     export CYLC_TASK_ID="${CYLC_TASK_NAME}.${CYLC_TASK_CYCLE_POINT}"
-    export CYLC_TASK_LOG_ROOT="${CYLC_SUITE_RUN_DIR}/log/job/${CYLC_TASK_JOB}/job"
+    export CYLC_TASK_LOG_DIR="${CYLC_SUITE_RUN_DIR}/log/job/${CYLC_TASK_JOB}"
+    export CYLC_TASK_LOG_ROOT="${CYLC_TASK_LOG_DIR}/job"
     if [[ -n "${CYLC_TASK_WORK_DIR_BASE:-}" ]]; then
         # Note: value of CYLC_TASK_WORK_DIR_BASE may contain variable
         # substitution syntax including some of the derived variables above, so
@@ -124,11 +125,8 @@ cylc__job__main() {
     # Send task started message
     cylc message -- "${CYLC_SUITE_NAME}" "${CYLC_TASK_JOB}" 'started' &
     CYLC_TASK_MESSAGE_STARTED_PID=$!
-    # Access to the suite bin directory
-    if [[ -n "${CYLC_SUITE_DEF_PATH:-}" && -d "${CYLC_SUITE_DEF_PATH}/bin" ]]
-    then
-        export PATH="${CYLC_SUITE_DEF_PATH}/bin:${PATH}"
-    fi
+    # Access to the suite bin directory (installed run-dir first).
+    export PATH="${CYLC_SUITE_RUN_DIR}/bin:${CYLC_SUITE_DEF_PATH}/bin:${PATH}"
     # Create share and work directories
     mkdir -p "${CYLC_SUITE_SHARE_DIR}" || true
     mkdir -p "$(dirname "${CYLC_TASK_WORK_DIR}")" || true
@@ -146,6 +144,8 @@ cylc__job__main() {
     wait "${CYLC_TASK_MESSAGE_STARTED_PID}" 2>'/dev/null' || true
     cylc message -- "${CYLC_SUITE_NAME}" "${CYLC_TASK_JOB}" 'succeeded' || true
     trap '' ${CYLC_VACATION_SIGNALS:-} ${CYLC_FAIL_SIGNALS}
+    # Execute success exit script
+    cylc__job__run_inst_func 'exit_script'
     exit 0
 }
 
@@ -177,7 +177,7 @@ cylc__job__run_inst_func() {
 #     see "cylc help message" for format of messages.
 # Returns:
 #   exit 1
-cylc__job_finish() {
+cylc__job_finish_err() {
     typeset signal="$1"
     typeset run_err_script="$2"
     shift 2
@@ -194,21 +194,21 @@ cylc__job_finish() {
 }
 
 ###############################################################################
-# Wrap cylc__job_finish to abort with a user-defined error message.
+# Wrap cylc__job_finish_err to abort with a user-defined error message.
 cylc__job_abort() {
-    cylc__job_finish "EXIT" true "CRITICAL: aborted/\"${1}\""
+    cylc__job_finish_err "EXIT" true "CRITICAL: aborted/\"${1}\""
 }
 
 ###############################################################################
-# Wrap cylc__job_finish for job preempt/vacation signal trap.
+# Wrap cylc__job_finish_err for job preempt/vacation signal trap.
 cylc__job_vacation() {
-    cylc__job_finish "${1}" false "WARNING: vacated/${1}"
+    cylc__job_finish_err "${1}" false "WARNING: vacated/${1}"
 }
 
 ###############################################################################
-# Wrap cylc__job_finish for automatic job exit signal trap.
+# Wrap cylc__job_finish_err for automatic job exit signal trap.
 cylc__job_err() {
-    cylc__job_finish "${1}" true "CRITICAL: failed/${1}"
+    cylc__job_finish_err "${1}" true "CRITICAL: failed/${1}"
 }
 
 ###############################################################################
