@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,6 +34,6 @@ PRIVILEGE_LEVELS = [
     PRIV_DESCRIPTION,
     PRIV_STATE_TOTALS,
     PRIV_FULL_READ,
-    PRIV_SHUTDOWN,  # (Not used yet - for the post-passhprase era.)
+    PRIV_SHUTDOWN,  # (Not used yet - for the post-passphrase era.)
     PRIV_FULL_CONTROL,
 ]
