@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@ import traceback
 from uuid import uuid4
 import warnings
 
+
 from cylc.cfgspec.glbl_cfg import glbl_cfg
 from cylc.exceptions import CylcError
 import cylc.flags
@@ -576,7 +577,7 @@ class SuiteRuntimeServiceClient(object):
             # * Add `-n` to the SSH command
             stdin = None
         proc = remote_cylc_cmd(
-            command, self.owner, self.host, capture=True,
+            command, self.owner, self.host, capture_process=True,
             ssh_login_shell=(self.comms1.get(
                 self.srv_files_mgr.KEY_SSH_USE_LOGIN_SHELL
             ) in ['True', 'true']),
@@ -710,49 +711,3 @@ def get_exception_from_html(html_text):
     except HTMLParseError:
         return None
     return parser.exception_text
-
-
-if __name__ == '__main__':
-    import unittest
-
-    class TestSuiteRuntimeServiceClient(unittest.TestCase):
-        """Unit testing class to test the methods in SuiteRuntimeServiceClient
-        """
-        def test_url_compiler_https(self):
-            """Tests that the url parser works for a single url and command
-            using https"""
-            myclient = SuiteRuntimeServiceClient(
-                "test-suite", host=get_host(), port=80)
-            myclient.comms1[SuiteSrvFilesManager.KEY_COMMS_PROTOCOL] = 'https'
-            self.assertEqual(
-                'https://%s:80/test_command?apples=False&oranges=True' %
-                get_host(),
-                myclient._call_server_get_url(
-                    "test_command", apples="False", oranges="True"))
-
-        def test_compile_url_compiler_http(self):
-            """Test that the url compiler produces a http request when
-            http is specified."""
-            myclient = SuiteRuntimeServiceClient(
-                "test-suite", host=get_host(), port=80)
-            myclient.comms1[SuiteSrvFilesManager.KEY_COMMS_PROTOCOL] = 'http'
-            self.assertEqual(
-                'http://%s:80/test_command?apples=False&oranges=True' %
-                get_host(),
-                myclient._call_server_get_url(
-                    "test_command", apples="False", oranges="True"))
-
-        def test_compile_url_compiler_none_specified(self):
-            """Test that the url compiler produces a http request when
-            none is specified. This should retrieve it from the
-            global config."""
-            myclient = SuiteRuntimeServiceClient(
-                "test-suite", host=get_host(), port=80)
-            url = myclient._call_server_get_url(
-                "test_command", apples="False", oranges="True")
-            # Check that the url has had http (or https) appended
-            # to it. (If it does not start with "http*" then something
-            # has gone wrong.)
-            self.assertTrue(url.startswith("http"))
-
-    unittest.main()
