@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,12 +19,12 @@
 
 import json
 import sqlite3
-import sys
 import traceback
 
+
+from cylc import LOG
 import cylc.flags
 from cylc.wallclock import get_current_time_string
-from cylc.suite_logging import LOG
 
 
 class CylcSuiteDAOTableColumn(object):
@@ -135,7 +135,7 @@ class CylcSuiteDAOTable(object):
     def add_update_item(self, set_args, where_args):
         """Queue an UPDATE item.
 
-        set_args should be a dict, with colum keys and values to be set.
+        set_args should be a dict, with column keys and values to be set.
         where_args should be a dict, update will only apply to rows matching
         all these items.
 
@@ -193,6 +193,7 @@ class CylcSuiteDAO(object):
     TABLE_TASK_POOL_CHECKPOINTS = "task_pool_checkpoints"
     TABLE_TASK_STATES = "task_states"
     TABLE_TASK_TIMEOUT_TIMERS = "task_timeout_timers"
+    TABLE_XTRIGGERS = "xtriggers"
 
     TABLES_ATTRS = {
         TABLE_BROADCAST_EVENTS: [
@@ -290,6 +291,10 @@ class CylcSuiteDAO(object):
             ["status"],
             ["hold_swap"],
         ],
+        TABLE_XTRIGGERS: [
+            ["signature", {"is_primary_key": True}],
+            ["results"],
+        ],
         TABLE_TASK_POOL_CHECKPOINTS: [
             ["id", {"datatype": "INTEGER", "is_primary_key": True}],
             ["cycle", {"is_primary_key": True}],
@@ -357,7 +362,7 @@ class CylcSuiteDAO(object):
     def add_update_item(self, table_name, set_args, where_args=None):
         """Queue an UPDATE item for a given table.
 
-        set_args should be a dict, with colum keys and values to be set.
+        set_args should be a dict, with column keys and values to be set.
         where_args should be a dict, update will only apply to rows matching
         all these items.
 
@@ -471,7 +476,19 @@ class CylcSuiteDAO(object):
             LOG.warning(err_log)
             raise
 
-    def select_broadcast_states(self, callback, id_key=None):
+    def pre_select_broadcast_states(self, id_key=None, order=None):
+        """Query statement and args formation for select_broadcast_states."""
+        form_stmt = r"SELECT point,namespace,key,value FROM %s"
+        if order == "ASC":
+            ordering = " ORDER BY point ASC, namespace ASC, key ASC"
+            form_stmt = form_stmt + ordering
+        if id_key is None or id_key == self.CHECKPOINT_LATEST_ID:
+            return form_stmt % self.TABLE_BROADCAST_STATES, []
+        else:
+            return (form_stmt % self.TABLE_BROADCAST_STATES_CHECKPOINTS +
+                    r" WHERE id==?"), [id_key]
+
+    def select_broadcast_states(self, callback, id_key=None, sort=None):
         """Select from broadcast_states or broadcast_states_checkpoints.
 
         Invoke callback(row_idx, row) on each row, where each row contains:
@@ -481,14 +498,27 @@ class CylcSuiteDAO(object):
         select from broadcast_states table if id_key == CHECKPOINT_LATEST_ID.
         Otherwise select from broadcast_states_checkpoints where id == id_key.
         """
-        form_stmt = r"SELECT point,namespace,key,value FROM %s"
-        if id_key is None or id_key == self.CHECKPOINT_LATEST_ID:
-            stmt = form_stmt % self.TABLE_BROADCAST_STATES
-            stmt_args = []
-        else:
-            stmt = (form_stmt % self.TABLE_BROADCAST_STATES_CHECKPOINTS +
-                    r" WHERE id==?")
-            stmt_args = [id_key]
+        stmt, stmt_args = self.pre_select_broadcast_states(id_key=None,
+                                                           order=sort)
+        for row_idx, row in enumerate(self.connect().execute(stmt, stmt_args)):
+            callback(row_idx, list(row))
+
+    def pre_select_broadcast_events(self, order=None):
+        """Query statement and args formation for select_broadcast_events."""
+        form_stmt = r"SELECT time,change,point,namespace,key,value FROM %s"
+        if order == "DESC":
+            ordering = (" ORDER BY " +
+                        "time DESC, point DESC, namespace DESC, key DESC")
+            form_stmt = form_stmt + ordering
+        return form_stmt % self.TABLE_BROADCAST_EVENTS, []
+
+    def select_broadcast_events(self, callback, id_key=None, sort=None):
+        """Select from broadcast_events.
+
+        Invoke callback(row_idx, row) on each row, where each row contains:
+            [time, change, point, namespace, key, value]
+        """
+        stmt, stmt_args = self.pre_select_broadcast_events(order=sort)
         for row_idx, row in enumerate(self.connect().execute(stmt, stmt_args)):
             callback(row_idx, list(row))
 
@@ -500,7 +530,7 @@ class CylcSuiteDAO(object):
 
         If id_key is specified, add where id == id_key to select.
         """
-        stmt = r"SELECT id,time,event FROM %s" % self.TABLE_CHECKPOINT_ID
+        stmt = r"SELECT id,time,event FROM checkpoint_id"
         stmt_args = []
         if id_key is not None:
             stmt += r" WHERE id==?"
@@ -509,6 +539,14 @@ class CylcSuiteDAO(object):
         for row_idx, row in enumerate(self.connect().execute(stmt, stmt_args)):
             callback(row_idx, list(row))
 
+    def select_checkpoint_id_restart_count(self):
+        """Return number of restart event in checkpoint_id table."""
+        stmt = r"SELECT COUNT(event) FROM checkpoint_id WHERE event==?"
+        stmt_args = ['restart']
+        for row in self.connect().execute(stmt, stmt_args):
+            return row[0]
+        return 0
+
     def select_suite_params(self, callback, id_key=None):
         """Select from suite_params or suite_params_checkpoints.
 
@@ -563,16 +601,15 @@ class CylcSuiteDAO(object):
         for row_idx, row in enumerate(self.connect().execute(stmt)):
             callback(row_idx, list(row))
 
-    def select_task_job(self, keys, cycle, name, submit_num=None):
+    def select_task_job(self, cycle, name, submit_num=None):
         """Select items from task_jobs by (cycle, name, submit_num).
 
-        Return a dict for mapping keys to the column values.
-
+        :return: a dict for mapping keys to the column values
+        :rtype: dict
         """
-        if keys is None:
-            keys = []
-            for column in self.tables[self.TABLE_TASK_JOBS].columns[3:]:
-                keys.append(column.name)
+        keys = []
+        for column in self.tables[self.TABLE_TASK_JOBS].columns[3:]:
+            keys.append(column.name)
         if submit_num in [None, "NN"]:
             stmt = (r"SELECT %(keys_str)s FROM %(table)s"
                     r" WHERE cycle==? AND name==?"
@@ -645,6 +682,11 @@ class CylcSuiteDAO(object):
             ret[(name, cycle)] = submit_num
         return ret
 
+    def select_xtriggers_for_restart(self, callback):
+        stm = r"SELECT signature,results FROM %s" % self.TABLE_XTRIGGERS
+        for row_idx, row in enumerate(self.connect().execute(stm, [])):
+            callback(row_idx, list(row))
+
     def select_task_pool(self, callback, id_key=None):
         """Select from task_pool or task_pool_checkpoints.
 
@@ -777,8 +819,7 @@ class CylcSuiteDAO(object):
         """
         id_ = 1
         for max_id, in self.connect().execute(
-                "SELECT MAX(id) FROM %(table)s" %
-                {"table": self.TABLE_CHECKPOINT_ID}):
+                "SELECT MAX(id) FROM checkpoint_id"):
             if max_id >= id_:
                 id_ = max_id + 1
         daos = [self]
@@ -801,7 +842,7 @@ class CylcSuiteDAO(object):
         conn = self.connect()
         # Add hold_swap column task_pool(_checkpoints) tables
         for t_name in [self.TABLE_TASK_POOL, self.TABLE_TASK_POOL_CHECKPOINTS]:
-            sys.stdout.write("Add hold_swap column to %s\n" % (t_name,))
+            LOG.info("Add hold_swap column to %s", t_name)
             conn.execute(
                 r"ALTER TABLE " + t_name + r" ADD COLUMN hold_swap TEXT")
         conn.commit()
@@ -819,7 +860,7 @@ class CylcSuiteDAO(object):
         if check_points:
             # No need to upgrade if latest check point already exists
             return
-        sys.stdout.write("Upgrading suite db with %s ...\n" % state_file_path)
+        LOG.info("Upgrading suite db with %s ...", state_file_path)
         self._upgrade_with_state_file_states(state_file_path)
         self._upgrade_with_state_file_extras()
 
@@ -829,7 +870,7 @@ class CylcSuiteDAO(object):
         Populate the new database tables with information from state file.
         """
         location = None
-        sys.stdout.write("Populating %s table" % self.TABLE_SUITE_PARAMS)
+        LOG.info("Populating %s table", self.TABLE_SUITE_PARAMS)
         for line in open(state_file_path):
             line = line.strip()
             if location is None:
@@ -840,11 +881,9 @@ class CylcSuiteDAO(object):
                 # The "broadcast_states" table should already be populated.
                 if line == "Begin task states":
                     location = "task states"
-                    sys.stdout.write(
-                        "\nPopulating %s table" % self.TABLE_TASK_POOL)
+                    LOG.info("Populating %s table", self.TABLE_TASK_POOL)
             else:
                 self._upgrade_with_state_file_tasks(line)
-        sys.stdout.write("\n")
         self.execute_queued_items()
 
     def _upgrade_with_state_file_header(self, line):
@@ -866,7 +905,7 @@ class CylcSuiteDAO(object):
                 self.add_insert_item(self.TABLE_SUITE_PARAMS, {
                     "key": key,
                     "value": tail})
-                sys.stdout.write("\n + %s=%s" % (key, tail))
+                LOG.info(" + %s=%s", key, tail)
                 if name == "final cycle":
                     return "broadcast"
                 else:
@@ -890,7 +929,7 @@ class CylcSuiteDAO(object):
             "spawned": spawned,
             "status": status,
             "hold_swap": None})
-        sys.stdout.write("\n + %s" % head)
+        LOG.info(" + %s", head)
 
     def _upgrade_with_state_file_extras(self):
         """Upgrade the database tables after reading in state file."""
@@ -908,7 +947,7 @@ class CylcSuiteDAO(object):
 
         # Populate new tables using old column data
         for t_name in [self.TABLE_TASK_STATES, self.TABLE_TASK_EVENTS]:
-            sys.stdout.write(r"Upgrading %s table " % (t_name))
+            LOG.info(r"Upgrading %s table", t_name)
             column_names = [col.name for col in self.tables[t_name].columns]
             for i, row in enumerate(conn.execute(
                     r"SELECT " + ",".join(column_names) +
@@ -916,10 +955,6 @@ class CylcSuiteDAO(object):
                 # These tables can be big, so we don't want to queue the items
                 # in memory.
                 conn.execute(self.tables[t_name].get_insert_stmt(), list(row))
-                if i:
-                    sys.stdout.write("\b" * len("%d rows" % (i)))
-                sys.stdout.write("%d rows" % (i + 1))
-            sys.stdout.write(" done\n")
         conn.commit()
 
         # Drop old tables
@@ -950,17 +985,16 @@ class CylcSuiteDAO(object):
         # Use of "pickle" module is for loading data written by <=7.6.X of Cylc
         # in users' own spaces.
         import pickle
-        sys.stdout.write(r"Upgrading %s table " % (t_name))
+        LOG.info(r"Upgrading %s table", t_name)
         cols = []
         for col in self.tables[t_name].columns:
             if col.name in ['ctx_key', 'ctx', 'delays']:
                 cols.append(col.name + '_pickle')
             else:
                 cols.append(col.name)
-        n_skips = 0
         # Codacy: Possible SQL injection vector through string-based query
         # construction.
-        # This is highly unlikely - all strings in the constuct are from
+        # This is highly unlikely - all strings in the construct are from
         # constants in this module.
         for i, row in enumerate(conn.execute(
                 r"SELECT " + ",".join(cols) + " FROM " + t_name + "_old")):
@@ -981,15 +1015,11 @@ class CylcSuiteDAO(object):
                     else:
                         args.append(cell)
             except (EOFError, TypeError, LookupError, ValueError):
-                n_skips += 1  # skip bad rows
+                pass
             else:
                 # These tables can be big, so we don't want to queue the items
                 # in memory.
                 conn.execute(self.tables[t_name].get_insert_stmt(), args)
-                if i:
-                    sys.stdout.write("\b" * len("%d rows" % (i)))
-                sys.stdout.write("%d rows" % (i + 1))
-        sys.stdout.write(" done, %d skipped\n" % n_skips)
         conn.commit()
 
         # Drop old tables
