@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,18 +25,18 @@ This module provides the logic to:
 * Manage existing run database files on restart.
 """
 
-import os
 import json
+import os
 from shutil import copy, rmtree
 from subprocess import call
 from tempfile import mkstemp
 
+
+from cylc import LOG
 from cylc.broadcast_report import get_broadcast_change_iter
-import cylc.flags
 from cylc.rundb import CylcSuiteDAO
-from cylc.suite_logging import ERR, LOG
 from cylc.version import CYLC_VERSION
-from cylc.wallclock import get_current_time_string
+from cylc.wallclock import get_current_time_string, get_utc_mode
 
 
 class SuiteDatabaseManager(object):
@@ -53,6 +53,7 @@ class SuiteDatabaseManager(object):
     TABLE_TASK_OUTPUTS = CylcSuiteDAO.TABLE_TASK_OUTPUTS
     TABLE_TASK_STATES = CylcSuiteDAO.TABLE_TASK_STATES
     TABLE_TASK_TIMEOUT_TIMERS = CylcSuiteDAO.TABLE_TASK_TIMEOUT_TIMERS
+    TABLE_XTRIGGERS = CylcSuiteDAO.TABLE_XTRIGGERS
 
     def __init__(self, pri_d=None, pub_d=None):
         self.pri_path = None
@@ -70,7 +71,8 @@ class SuiteDatabaseManager(object):
             self.TABLE_TASK_POOL: [],
             self.TABLE_TASK_ACTION_TIMERS: [],
             self.TABLE_TASK_OUTPUTS: [],
-            self.TABLE_TASK_TIMEOUT_TIMERS: []}
+            self.TABLE_TASK_TIMEOUT_TIMERS: [],
+            self.TABLE_XTRIGGERS: []}
         self.db_inserts_map = {
             self.TABLE_BROADCAST_EVENTS: [],
             self.TABLE_BROADCAST_STATES: [],
@@ -81,7 +83,8 @@ class SuiteDatabaseManager(object):
             self.TABLE_TASK_POOL: [],
             self.TABLE_TASK_ACTION_TIMERS: [],
             self.TABLE_TASK_OUTPUTS: [],
-            self.TABLE_TASK_TIMEOUT_TIMERS: []}
+            self.TABLE_TASK_TIMEOUT_TIMERS: [],
+            self.TABLE_XTRIGGERS: []}
         self.db_updates_map = {}
 
     def checkpoint(self, name):
@@ -272,7 +275,7 @@ class SuiteDatabaseManager(object):
              "value": schd.task_job_mgr.task_remote_mgr.uuid_str},
             {"key": "run_mode", "value": schd.run_mode},
             {"key": "cylc_version", "value": CYLC_VERSION},
-            {"key": "UTC_mode", "value": cylc.flags.utc},
+            {"key": "UTC_mode", "value": get_utc_mode()},
             {"key": "initial_point", "value": str(schd.initial_point)},
             {"key": "final_point", "value": final_point_str},
         ])
@@ -312,6 +315,14 @@ class SuiteDatabaseManager(object):
                     "delay": timer.delay,
                     "timeout": timer.timeout})
 
+    def put_xtriggers(self, sat_xtrig):
+        """Put statements to update external triggers table."""
+        self.db_deletes_map[self.TABLE_XTRIGGERS].append({})
+        for sig, res in sat_xtrig.items():
+            self.db_inserts_map[self.TABLE_XTRIGGERS].append({
+                "signature": sig,
+                "results": json.dumps(res)})
+
     def put_task_pool(self, pool):
         """Put statements to update the task_pool table in runtime database.
 
@@ -470,7 +481,7 @@ class SuiteDatabaseManager(object):
                     os.unlink(os.path.join(suite_run_d, "state.tar.gz"))
                 except OSError:
                     pass
-                ERR.error("cannot tar-gzip + remove old state/ directory")
+                LOG.error("cannot tar-gzip + remove old state/ directory")
             # Remove old files as well
             try:
                 os.unlink(os.path.join(suite_run_d, "cylc-suite-env"))
