@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,16 +33,13 @@ import os
 from pipes import quote
 import shlex
 from time import time
-import traceback
 
 from parsec.config import ItemNotFoundError
 
-from cylc.broadcast_mgr import BroadcastMgr
+from cylc import LOG
 from cylc.cfgspec.glbl_cfg import glbl_cfg
-import cylc.flags
-from cylc.mp_pool import SuiteProcContext
-from cylc.suite_logging import ERR, LOG
 from cylc.hostuserutil import get_host, get_user
+from cylc.subprocctx import SubProcContext
 from cylc.task_action_timer import TaskActionTimer
 from cylc.task_job_logs import (
     get_task_job_log, get_task_job_activity_log, JOB_LOG_OUT, JOB_LOG_ERR)
@@ -59,7 +56,6 @@ from cylc.task_outputs import (
 from cylc.wallclock import (
     get_current_time_string, get_seconds_as_interval_string as intvl_as_str)
 
-
 CustomTaskEventHandlerContext = namedtuple(
     "CustomTaskEventHandlerContext",
     ["key", "ctx_type", "cmd"])
@@ -91,7 +87,7 @@ def log_task_job_activity(ctx, suite, point, name, submit_num=None):
         # This happens when there is no job directory, e.g. if job host
         # selection command causes an submission failure, there will be no job
         # directory. In this case, just send the information to the suite log.
-        LOG.debug(exc)
+        LOG.exception(exc)
         LOG.info(ctx_str)
     if ctx.cmd and ctx.ret_code:
         LOG.error(ctx_str)
@@ -130,16 +126,15 @@ class TaskEventsManager(object):
         "CRITICAL": CRITICAL,
         "DEBUG": DEBUG,
     }
+    NON_UNIQUE_EVENTS = ('warning', 'critical', 'custom')
     POLLED_FLAG = "(polled)"
 
-    def __init__(self, suite, proc_pool, suite_db_mgr, broadcast_mgr=None):
+    def __init__(self, suite, proc_pool, suite_db_mgr, broadcast_mgr):
         self.suite = suite
         self.suite_url = None
         self.suite_cfg = {}
         self.proc_pool = proc_pool
         self.suite_db_mgr = suite_db_mgr
-        if broadcast_mgr is None:
-            broadcast_mgr = BroadcastMgr(self.suite_db_mgr)
         self.broadcast_mgr = broadcast_mgr
         self.mail_interval = 0.0
         self.mail_footer = None
@@ -148,7 +143,7 @@ class TaskEventsManager(object):
         # Set pflag = True to stimulate task dependency negotiation whenever a
         # task changes state in such a way that others could be affected. The
         # flag should only be turned off again after use in
-        # Scheduler.process_tasks, to ensure that dependency negotation occurs
+        # Scheduler.process_tasks, to ensure that dependency negotiation occurs
         # when required.
         self.pflag = False
 
@@ -197,7 +192,7 @@ class TaskEventsManager(object):
             pass
         itask.timeout = None  # emit event only once
         if msg and event:
-            LOG.warning(msg, itask=itask)
+            LOG.warning('[%s] -%s', itask, msg)
             self.setup_event_handlers(itask, event, msg)
             return True
         else:
@@ -221,7 +216,7 @@ class TaskEventsManager(object):
     def process_events(self, schd_ctx):
         """Process task events that were created by "setup_event_handlers".
 
-        schd_ctx is an instance of "Schduler" in "cylc.scheduler".
+        schd_ctx is an instance of "Scheduler" in "cylc.scheduler".
         """
         ctx_groups = {}
         now = time()
@@ -262,7 +257,7 @@ class TaskEventsManager(object):
             if timer.ctx.ctx_type == self.HANDLER_CUSTOM:
                 # Run custom event handlers on their own
                 self.proc_pool.put_command(
-                    SuiteProcContext(
+                    SubProcContext(
                         (key1, submit_num),
                         timer.ctx.cmd, env=os.environ, shell=True,
                     ),
@@ -330,24 +325,24 @@ class TaskEventsManager(object):
             submit_num = itask.submit_num
         if flag == self.INCOMING_FLAG and submit_num != itask.submit_num:
             flag = self.IGNORED_INCOMING_FLAG
-        log_message = '(current:%s)%s %s at %s' % (
-            itask.state.status, flag, message, event_time)
-        LOG.log(self.LEVELS.get(severity, INFO), log_message, itask=itask)
+        LOG.log(
+            self.LEVELS.get(severity, INFO),
+            r'[%s] -(current:%s)%s %s at %s',
+            itask, itask.state.status, flag, message, event_time)
         if flag == self.IGNORED_INCOMING_FLAG:
             LOG.warning(
-                'submit-num=%d: ignore message from job submit-num=%d' % (
-                    itask.submit_num, submit_num),
-                itask=itask)
+                '[%s] -submit-num=%02d: ignore message from job(%02d)',
+                itask, itask.submit_num, submit_num)
             return
 
         # always update the suite state summary for latest message
-        itask.summary['latest_message'] = message
         if flag == self.POLLED_FLAG:
-            itask.summary['latest_message'] += ' %s' % self.POLLED_FLAG
-        cylc.flags.iflag = True
+            itask.set_summary_message('%s %s' % (message, self.POLLED_FLAG))
+        else:
+            itask.set_summary_message(message)
 
         # Satisfy my output, if possible, and record the result.
-        an_output_was_satisfied = itask.state.outputs.set_msg_trg_completion(
+        completed_trigger = itask.state.outputs.set_msg_trg_completion(
             message=message, is_completed=True)
 
         if message == TASK_OUTPUT_STARTED:
@@ -403,11 +398,17 @@ class TaskEventsManager(object):
             self.pflag = True
             itask.state.reset_state(TASK_STATUS_SUBMITTED)
             self._reset_job_timers(itask)
-        elif an_output_was_satisfied:
+            # We should really have a special 'vacated' handler, but given that
+            # this feature can only be used on the deprecated loadleveler
+            # system, we should probably aim to remove support for job vacation
+            # instead. Otherwise, we should have:
+            # self.setup_event_handlers(itask, 'vacated', message)
+        elif completed_trigger:
             # Message of an as-yet unreported custom task output.
             # No state change.
             self.pflag = True
             self.suite_db_mgr.put_update_task_outputs(itask)
+            self.setup_event_handlers(itask, completed_trigger, message)
         else:
             # Unhandled messages. These include:
             #  * general non-output/progress messages
@@ -415,14 +416,17 @@ class TaskEventsManager(object):
             # Note that all messages are logged already at the top.
             # No state change.
             LOG.debug(
-                '(current:%s) unhandled: %s' % (itask.state.status, message),
-                itask=itask)
+                '[%s] -(current:%s) unhandled: %s',
+                itask, itask.state.status, message)
             if severity in [CRITICAL, ERROR, WARNING, INFO, DEBUG]:
                 severity = getLevelName(severity)
             self._db_events_insert(
                 itask, ("message %s" % str(severity).lower()), message)
-        if severity in ['WARNING', 'CRITICAL', 'CUSTOM']:
-            self.setup_event_handlers(itask, severity.lower(), message)
+        lseverity = str(severity).lower()
+        if lseverity in self.NON_UNIQUE_EVENTS:
+            itask.non_unique_events.setdefault(lseverity, 0)
+            itask.non_unique_events[lseverity] += 1
+            self.setup_event_handlers(itask, lseverity, message)
 
     def setup_event_handlers(self, itask, event, message):
         """Set up handlers for a task event."""
@@ -500,7 +504,7 @@ class TaskEventsManager(object):
         if mail_smtp:
             env["smtp"] = mail_smtp
         self.proc_pool.put_command(
-            SuiteProcContext(
+            SubProcContext(
                 ctx, cmd, env=env, stdin_str=stdin_str, id_keys=id_keys,
             ),
             self._event_email_callback, [schd_ctx])
@@ -512,15 +516,14 @@ class TaskEventsManager(object):
             try:
                 if proc_ctx.ret_code == 0:
                     del self.event_timers[id_key]
-                    log_ctx = SuiteProcContext((key1, submit_num), None)
+                    log_ctx = SubProcContext((key1, submit_num), None)
                     log_ctx.ret_code = 0
                     log_task_job_activity(
                         log_ctx, schd_ctx.suite, point, name, submit_num)
                 else:
                     self.event_timers[id_key].unset_waiting()
-            except KeyError:
-                if cylc.flags.debug:
-                    ERR.debug(traceback.format_exc())
+            except KeyError as exc:
+                LOG.exception(exc)
 
     def _get_events_conf(self, itask, key, default=None):
         """Return an events setting from suite then global configuration."""
@@ -548,7 +551,7 @@ class TaskEventsManager(object):
             "retrieve job logs command", s_host, s_user))
 
         cmd = shlex.split(rsync_str) + ["--rsh=" + ssh_str]
-        if cylc.flags.debug:
+        if LOG.isEnabledFor(DEBUG):
             cmd.append("-v")
         if ctx.max_size:
             cmd.append("--max-size=%s" % (ctx.max_size,))
@@ -569,7 +572,7 @@ class TaskEventsManager(object):
         cmd.append(glbl_cfg().get_derived_host_item(
             schd_ctx.suite, "suite job log directory") + "/")
         self.proc_pool.put_command(
-            SuiteProcContext(ctx, cmd, env=dict(os.environ), id_keys=id_keys),
+            SubProcContext(ctx, cmd, env=dict(os.environ), id_keys=id_keys),
             self._job_logs_retrieval_callback, [schd_ctx])
 
     def _job_logs_retrieval_callback(self, proc_ctx, schd_ctx):
@@ -589,7 +592,7 @@ class TaskEventsManager(object):
                     fname_oks[fname] = os.path.exists(get_task_job_log(
                         schd_ctx.suite, point, name, submit_num, fname))
                 # All expected paths must exist to record a good attempt
-                log_ctx = SuiteProcContext((key1, submit_num), None)
+                log_ctx = SubProcContext((key1, submit_num), None)
                 if all(fname_oks.values()):
                     log_ctx.ret_code = 0
                     del self.event_timers[id_key]
@@ -602,9 +605,8 @@ class TaskEventsManager(object):
                     self.event_timers[id_key].unset_waiting()
                 log_task_job_activity(
                     log_ctx, schd_ctx.suite, point, name, submit_num)
-            except KeyError:
-                if cylc.flags.debug:
-                    ERR.debug(traceback.format_exc())
+            except KeyError as exc:
+                LOG.exception(exc)
 
     def _process_message_failed(self, itask, event_time, message):
         """Helper for process_message, handle a failed message."""
@@ -621,15 +623,15 @@ class TaskEventsManager(object):
             self.pflag = True
             if itask.state.reset_state(TASK_STATUS_FAILED):
                 self.setup_event_handlers(itask, "failed", message)
-            LOG.critical("job(%02d) %s" % (
-                itask.submit_num, "failed"), itask=itask)
+            LOG.critical(
+                "[%s] -job(%02d) %s", itask, itask.submit_num, "failed")
         else:
             # There is a retry lined up
             delay_msg = "retrying in %s" % (
                 itask.try_timers[TASK_STATUS_RETRYING].delay_timeout_as_str())
             msg = "failed, %s" % (delay_msg)
-            LOG.info("job(%02d) %s" % (itask.submit_num, msg), itask=itask)
-            itask.summary['latest_message'] = msg
+            LOG.info("[%s] -job(%02d) %s", itask, itask.submit_num, msg)
+            itask.set_summary_message(msg)
             if itask.state.reset_state(TASK_STATUS_RETRYING):
                 self.setup_event_handlers(
                     itask, "retry", "%s, %s" % (self.JOB_FAILED, delay_msg))
@@ -639,7 +641,7 @@ class TaskEventsManager(object):
         """Helper for process_message, handle a started message."""
         if itask.job_vacated:
             itask.job_vacated = False
-            LOG.warning("Vacated job restarted", itask=itask)
+            LOG.warning("[%s] -Vacated job restarted", itask)
         self.pflag = True
         if itask.state.reset_state(TASK_STATUS_RUNNING):
             self.setup_event_handlers(itask, 'started', 'job started')
@@ -673,15 +675,16 @@ class TaskEventsManager(object):
                                   TASK_OUTPUT_FAILED]:
                     msg += "\n  " + output
             if msg:
-                LOG.info("Succeeded with outputs not completed: %s" % msg,
-                         itask=itask)
+                LOG.info(
+                    "[%s] -Succeeded with outputs not completed: %s",
+                    itask, msg)
         if itask.state.reset_state(TASK_STATUS_SUCCEEDED):
             self.setup_event_handlers(itask, "succeeded", "job succeeded")
         self._reset_job_timers(itask)
 
     def _process_message_submit_failed(self, itask, event_time):
         """Helper for process_message, handle a submit-failed message."""
-        LOG.error(self.EVENT_SUBMIT_FAILED, itask=itask)
+        LOG.error('[%s] -%s', itask, self.EVENT_SUBMIT_FAILED)
         if event_time is None:
             event_time = get_current_time_string()
         self.suite_db_mgr.put_update_task_jobs(itask, {
@@ -703,8 +706,8 @@ class TaskEventsManager(object):
             timer = itask.try_timers[TASK_STATUS_SUBMIT_RETRYING]
             delay_msg = "submit-retrying in %s" % timer.delay_timeout_as_str()
             msg = "%s, %s" % (self.EVENT_SUBMIT_FAILED, delay_msg)
-            LOG.info("job(%02d) %s" % (itask.submit_num, msg), itask=itask)
-            itask.summary['latest_message'] = msg
+            LOG.info("[%s] -job(%02d) %s", itask, itask.submit_num, msg)
+            itask.set_summary_message(msg)
             if itask.state.reset_state(TASK_STATUS_SUBMIT_RETRYING):
                 self.setup_event_handlers(
                     itask, self.EVENT_SUBMIT_RETRY,
@@ -715,10 +718,12 @@ class TaskEventsManager(object):
         """Helper for process_message, handle a submit-succeeded message."""
         try:
             LOG.info(
-                ('job[%(submit_num)02d] submitted to'
-                 ' %(host)s:%(batch_sys_name)s[%(submit_method_id)s]') %
-                itask.summary,
-                itask=itask)
+                '[%s] -job[%02d] submitted to %s:%s[%s]',
+                itask,
+                itask.summary['submit_num'],
+                itask.summary['host'],
+                itask.summary['batch_sys_name'],
+                itask.summary['submit_method_id'])
         except KeyError:
             pass
         self.suite_db_mgr.put_update_task_jobs(itask, {
@@ -738,7 +743,7 @@ class TaskEventsManager(object):
         # Unset started and finished times in case of resubmission.
         itask.set_summary_time('started')
         itask.set_summary_time('finished')
-        itask.summary['latest_message'] = TASK_OUTPUT_SUBMITTED
+        itask.set_summary_message(TASK_OUTPUT_SUBMITTED)
 
         self.pflag = True
         if itask.state.status == TASK_STATUS_READY:
@@ -783,9 +788,13 @@ class TaskEventsManager(object):
 
     def _setup_event_mail(self, itask, event):
         """Set up task event notification, by email."""
-        id_key = (
-            (self.HANDLER_MAIL, event),
-            str(itask.point), itask.tdef.name, itask.submit_num)
+        if event in self.NON_UNIQUE_EVENTS:
+            key1 = (
+                self.HANDLER_MAIL,
+                '%s-%d' % (event, itask.non_unique_events.get(event, 1)))
+        else:
+            key1 = (self.HANDLER_MAIL, event)
+        id_key = (key1, str(itask.point), itask.tdef.name, itask.submit_num)
         if (id_key in self.event_timers or
                 event not in self._get_events_conf(itask, "mail events", [])):
             return
@@ -822,7 +831,12 @@ class TaskEventsManager(object):
             retry_delays = [0]
         # There can be multiple custom event handlers
         for i, handler in enumerate(handlers):
-            key1 = ("%s-%02d" % (self.HANDLER_CUSTOM, i), event)
+            if event in self.NON_UNIQUE_EVENTS:
+                key1 = (
+                    '%s-%02d' % (self.HANDLER_CUSTOM, i),
+                    '%s-%d' % (event, itask.non_unique_events.get(event, 1)))
+            else:
+                key1 = ('%s-%02d' % (self.HANDLER_CUSTOM, i), event)
             id_key = (
                 key1, str(itask.point), itask.tdef.name, itask.submit_num)
             if id_key in self.event_timers:
@@ -844,6 +858,7 @@ class TaskEventsManager(object):
                     "point": quote(str(itask.point)),
                     "name": quote(itask.tdef.name),
                     "submit_num": itask.submit_num,
+                    "try_num": itask.get_try_num(),
                     "id": quote(itask.identity),
                     "message": quote(message),
                     "batch_sys_name": quote(
@@ -883,7 +898,7 @@ class TaskEventsManager(object):
                 # Nothing substituted, assume classic interface
                 cmd = "%s '%s' '%s' '%s' '%s'" % (
                     handler, event, self.suite, itask.identity, message)
-            LOG.debug("Queueing %s handler: %s" % (event, cmd), itask=itask)
+            LOG.debug("[%s] -Queueing %s handler: %s", itask, event, cmd)
             self.event_timers[id_key] = (
                 TaskActionTimer(
                     CustomTaskEventHandlerContext(
@@ -924,7 +939,7 @@ class TaskEventsManager(object):
                 time_limit_delays = batch_sys_conf.get(
                     'execution time limit polling intervals', [60, 120, 420])
                 timeout = time_limit + sum(time_limit_delays)
-                # Remove execessive polling before time limit
+                # Remove excessive polling before time limit
                 while sum(delays) > time_limit:
                     del delays[-1]
                 # But fill up the gap before time limit
@@ -949,7 +964,7 @@ class TaskEventsManager(object):
         itask.poll_timer = TaskActionTimer(ctx=ctx, delays=delays)
         # Log timeout and polling schedule
         message = 'health check settings: %s=%s' % (timeout_key, timeout_str)
-        # Attempt to group idenitical consecutive delays as N*DELAY,...
+        # Attempt to group identical consecutive delays as N*DELAY,...
         if itask.poll_timer.delays:
             items = []  # [(number of item - 1, item), ...]
             for delay in itask.poll_timer.delays:
@@ -963,6 +978,6 @@ class TaskEventsManager(object):
                     message += '%d*' % (num + 1)
                 message += '%s,' % intvl_as_str(item)
             message += '...'
-        LOG.info(message, itask=itask)
+        LOG.info('[%s] -%s', itask, message)
         # Set next poll time
         self.check_poll_time(itask)
