@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -62,6 +62,8 @@ class TaskProxy(object):
         .manual_trigger (boolean):
             Has this task received a manual trigger command? This flag is reset
             on trigger.
+        .non_unique_events (dict):
+            Count non-unique events (e.g. critical, warning, custom).
         .point (cylc.cycling.PointBase):
             Cycle point of the task.
         .point_as_seconds (int):
@@ -157,6 +159,7 @@ class TaskProxy(object):
         'late_time',
         'local_job_file_path',
         'manual_trigger',
+        'non_unique_events',
         'point',
         'point_as_seconds',
         'poll_timer',
@@ -210,15 +213,10 @@ class TaskProxy(object):
             'latest_message': '',
             'submitted_time': None,
             'submitted_time_string': None,
-            'submit_num': self.submit_num,
             'started_time': None,
             'started_time_string': None,
             'finished_time': None,
             'finished_time_string': None,
-            'name': self.tdef.name,
-            'description': self.tdef.rtconfig['meta']['description'],
-            'title': self.tdef.rtconfig['meta']['title'],
-            'label': str(self.point),
             'logfiles': [],
             'job_hosts': {},
             'execution_time_limit': None,
@@ -235,6 +233,9 @@ class TaskProxy(object):
         self.poll_timer = None
         self.timeout = None
         self.try_timers = {}
+        # Use dict here for Python 2.6 compat.
+        # Should use collections.Counter in Python 2.7+
+        self.non_unique_events = {}
 
         self.clock_trigger_time = None
         self.expire_time = None
@@ -258,6 +259,10 @@ class TaskProxy(object):
                         self.cleanup_cutoff < p_next):
                     self.cleanup_cutoff = p_next
 
+    def __str__(self):
+        """Stringify using "self.identity"."""
+        return self.identity
+
     def copy_pre_reload(self, pre_reload_inst):
         """Copy attributes from pre-reload instant."""
         self.submit_num = pre_reload_inst.submit_num
@@ -308,19 +313,24 @@ class TaskProxy(object):
 
     def get_state_summary(self):
         """Return a dict containing the state summary of this task proxy."""
-        self.summary['state'] = self.state.status
-        self.summary['spawned'] = str(self.has_spawned)
-        count = len(self.tdef.elapsed_times)
-        if count:
-            self.summary['mean_elapsed_time'] = (
-                float(sum(self.tdef.elapsed_times)) / count)
-        elif self.summary['execution_time_limit']:
-            self.summary['mean_elapsed_time'] = float(
-                self.summary['execution_time_limit'])
+        ret = self.summary.copy()
+        ret['name'] = self.tdef.name
+        ret['description'] = self.tdef.rtconfig['meta']['description']
+        ret['title'] = self.tdef.rtconfig['meta']['title']
+        ret['label'] = str(self.point)
+        ret['submit_num'] = self.submit_num
+        ret['state'] = self.state.status
+        ret['spawned'] = str(self.has_spawned)
+        ntimes = len(self.tdef.elapsed_times)
+        if ntimes:
+            ret['mean_elapsed_time'] = (
+                float(sum(self.tdef.elapsed_times)) / ntimes)
+        elif ret['execution_time_limit']:
+            ret['mean_elapsed_time'] = float(
+                ret['execution_time_limit'])
         else:
-            self.summary['mean_elapsed_time'] = None
-
-        return self.summary
+            ret['mean_elapsed_time'] = None
+        return ret
 
     def get_try_num(self):
         """Return the number of automatic tries (try number)."""
@@ -366,6 +376,15 @@ class TaskProxy(object):
             for timer in self.try_timers.values():
                 timer.timeout = None
 
+    def set_summary_message(self, message):
+        """Set `.summary['latest_message']` if necessary.
+
+        Set `.state.is_updated` to `True` if message is updated.
+        """
+        if self.summary['latest_message'] != message:
+            self.summary['latest_message'] = message
+            self.state.is_updated = True
+
     def set_summary_time(self, event_key, time_str=None):
         """Set an event time in self.summary
 
@@ -390,8 +409,9 @@ class TaskProxy(object):
     def is_waiting_prereqs(self):
         """Is this task waiting for its prerequisites?"""
         return (
-            any(not pre.is_satisfied() for pre in self.state.prerequisites) or
-            any(not tri for tri in self.state.external_triggers.values())
+            any(not pre.is_satisfied() for pre in self.state.prerequisites)
+            or any(not tri for tri in self.state.external_triggers.values())
+            or not self.state.xtriggers_all_satisfied()
         )
 
     def is_waiting_retry(self, now):
