@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,11 +33,11 @@ from tempfile import NamedTemporaryFile
 from time import time
 from uuid import uuid4
 
+from cylc import LOG
 from cylc.cfgspec.glbl_cfg import glbl_cfg
 import cylc.flags
 from cylc.hostuserutil import is_remote, is_remote_host, is_remote_user
-from cylc.mp_pool import SuiteProcContext
-from cylc.suite_logging import LOG
+from cylc.subprocctx import SubProcContext
 from cylc.task_remote_cmd import (
     FILE_BASE_UUID, REMOTE_INIT_DONE, REMOTE_INIT_NOT_REQUIRED)
 
@@ -124,7 +124,7 @@ class TaskRemoteMgr(object):
                 else:
                     cmd = ['bash', '-c', cmd_str]
                 self.proc_pool.put_command(
-                    SuiteProcContext(
+                    SubProcContext(
                         'remote-host-select', cmd, env=dict(os.environ)),
                     self._remote_host_select_callback, [cmd_str])
                 self.remote_host_str_map[cmd_str] = None
@@ -224,7 +224,7 @@ class TaskRemoteMgr(object):
         cmd.append(glbl_cfg().get_derived_host_item(
             self.suite, 'suite run directory', host, owner))
         self.proc_pool.put_command(
-            SuiteProcContext(
+            SubProcContext(
                 'remote-init', cmd, stdin_file_paths=[tmphandle.name]),
             self._remote_init_callback,
             [host, owner, tmphandle])
@@ -290,7 +290,7 @@ class TaskRemoteMgr(object):
                 LOG.warning(TaskRemoteMgmtError(
                     TaskRemoteMgmtError.MSG_TIDY,
                     (host, owner), ' '.join(quote(item) for item in cmd),
-                    proc.ret_code, out, err))
+                    proc.returncode, out, err))
 
     def _remote_host_select_callback(self, proc_ctx, cmd_str):
         """Callback when host select command exits"""
@@ -309,7 +309,10 @@ class TaskRemoteMgr(object):
     def _remote_init_callback(self, proc_ctx, host, owner, tmphandle):
         """Callback when "cylc remote-init" exits"""
         self.ready = True
-        tmphandle.close()
+        try:
+            tmphandle.close()
+        except OSError:  # E.g. ignore bad unlink, etc
+            pass
         if proc_ctx.ret_code == 0:
             for status in (REMOTE_INIT_DONE, REMOTE_INIT_NOT_REQUIRED):
                 if status in proc_ctx.out:
