@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,9 +19,8 @@
 """Task state related logic."""
 
 
-import cylc.flags as flags
+from cylc import LOG
 from cylc.prerequisite import Prerequisite
-from cylc.suite_logging import LOG
 from cylc.task_id import TaskID
 from cylc.task_outputs import (
     TaskOutputs,
@@ -110,12 +109,15 @@ TASK_STATUSES_CAN_RESET_TO = set([
 ])
 
 # Task statuses that are final.
-TASK_STATUSES_FINAL = set([
+TASK_STATUSES_SUCCESS = set([
     TASK_STATUS_EXPIRED,
-    TASK_STATUS_SUCCEEDED,
-    TASK_STATUS_FAILED,
-    TASK_STATUS_SUBMIT_FAILED,
+    TASK_STATUS_SUCCEEDED
 ])
+TASK_STATUSES_FAILURE = set([
+    TASK_STATUS_FAILED,
+    TASK_STATUS_SUBMIT_FAILED
+])
+TASK_STATUSES_FINAL = TASK_STATUSES_SUCCESS | TASK_STATUSES_FAILURE
 
 # Task statuses that are never active.
 # For tasks that have never been submitted, but excluding:
@@ -158,7 +160,7 @@ TASK_STATUSES_TRIGGERABLE = set([
     TASK_STATUS_RETRYING
 ])
 
-# Tasks statuses to auto-exand in the gcylc tree view.
+# Tasks statuses to auto-expand in the gcylc tree view.
 TASK_STATUSES_AUTO_EXPAND = set([
     TASK_STATUS_QUEUED,
     TASK_STATUS_READY,
@@ -168,6 +170,13 @@ TASK_STATUSES_AUTO_EXPAND = set([
     TASK_STATUS_FAILED,
 ])
 
+# Tasks statuses grouped for cylc review 'create_suite_cycles_summary()'
+TASK_STATUS_GROUPS = {
+    "active": list(TASK_STATUSES_NOT_STALLED),
+    "fail": list(TASK_STATUSES_FAILURE),
+    "success": list(TASK_STATUSES_SUCCESS)
+}
+
 
 def status_leq(status_a, status_b):
     """"Return True if status_a <= status_b"""
@@ -182,18 +191,68 @@ def status_geq(status_a, status_b):
 
 
 class TaskState(object):
-    """Task status and utilities."""
+    """Task status and utilities.
+
+    Attributes:
+        .external_triggers (dict):
+            External triggers as {trigger (str): satisfied (boolean), ...}.
+        .hold_swap (str):
+            While the task is in `held` status, this holds the actual status if
+            the task is not held. For tasks in `submitted` or `running`
+            statuses, setting this to `held` will cause the task to hold when
+            the task is reset to anything other than the `submitted` or
+            `running` statuses.
+        .identity (str):
+            The task ID as `TASK.CYCLE` associated with this object.
+        .is_updated (boolean):
+            Has the status been updated since previous update?
+        .kill_failed (boolean):
+            Has a job kill attempt failed since previous status change?
+        .outputs (cylc.task_outputs.TaskOutputs):
+            Known outputs of the task.
+        .prerequisites (list<cylc.prerequisite.Prerequisite>):
+            List of prerequisites of the task.
+        .status (str):
+            The current status of the task.
+        .suicide_prerequisites (list<cylc.prerequisite.Prerequisite>):
+            List of prerequisites that will cause the task to suicide.
+        .time_updated (str):
+            Time string of latest update time.
+        .xclock (tuple):
+            A tuple (clock_label (str), is_done (boolean)) to indicate if a
+            clock trigger is satisfied or not. Set to `None` if the task has no
+            clock trigger.
+        .xtriggers (dict):
+            xtriggers as {trigger (str): satisfied (boolean), ...}.
+        ._is_satisfied (boolean):
+            Are prerequisites satisified?
+        ._suicide_is_satisfied (boolean):
+            Are prerequisites to trigger suicide satisified?
+    """
 
     # Memory optimization - constrain possible attributes to this list.
-    __slots__ = ["identity", "status", "hold_swap",
-                 "_is_satisfied", "_suicide_is_satisfied", "prerequisites",
-                 "suicide_prerequisites", "external_triggers", "outputs",
-                 "kill_failed", "time_updated"]
+    __slots__ = [
+        "external_triggers",
+        "hold_swap",
+        "identity",
+        "is_updated",
+        "kill_failed",
+        "outputs",
+        "prerequisites",
+        "status",
+        "suicide_prerequisites",
+        "time_updated",
+        "xclock",
+        "xtriggers",
+        "_is_satisfied",
+        "_suicide_is_satisfied",
+    ]
 
     def __init__(self, tdef, point, status, hold_swap):
         self.identity = TaskID.get(tdef.name, str(point))
         self.status = status
         self.hold_swap = hold_swap
+        self.is_updated = False
         self.time_updated = None
 
         self._is_satisfied = None
@@ -213,9 +272,26 @@ class TaskState(object):
             # set unsatisfied
             self.external_triggers[ext] = False
 
+        # xtriggers (represented by labels) satisfied or not
+        self.xtriggers = {}
+        for label in tdef.xtrig_labels:
+            self.xtriggers[label] = False
+        if tdef.xclock_label:
+            self.xclock = (tdef.xclock_label, False)
+        else:
+            self.xclock = None
+
+        # Message outputs.
         self.outputs = TaskOutputs(tdef)
         self.kill_failed = False
 
+    def __str__(self):
+        """Print status (hold_swap)."""
+        ret = self.status
+        if self.hold_swap:
+            ret += ' (%s)' % self.hold_swap
+        return ret
+
     def satisfy_me(self, all_task_outputs):
         """Attempt to get my prerequisites satisfied."""
         for prereqs in [self.prerequisites, self.suicide_prerequisites]:
@@ -224,6 +300,12 @@ class TaskState(object):
                     self._is_satisfied = None
                     self._suicide_is_satisfied = None
 
+    def xtriggers_all_satisfied(self):
+        """Return True if xclock and all xtriggers are satisfied."""
+        if self.xclock is not None and not self.xclock[1]:
+            return False
+        return all(self.xtriggers.values())
+
     def prerequisites_are_all_satisfied(self):
         """Return True if (non-suicide) prerequisites are fully satisfied."""
         if self._is_satisfied is None:
@@ -388,7 +470,7 @@ class TaskState(object):
             self.hold_swap = None
         self.status = status
         self.time_updated = get_current_time_string()
-        flags.iflag = True
+        self.is_updated = True
         # Log
         message = str(prev_status)
         if prev_hold_swap:
@@ -396,7 +478,7 @@ class TaskState(object):
         message += " => %s" % self.status
         if self.hold_swap:
             message += " (%s)" % self.hold_swap
-        LOG.debug(message, itask=self.identity)
+        LOG.debug("[%s] -%s", self.identity, message)
         return (prev_status, prev_hold_swap)
 
     def is_gt(self, status):
