@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2018 NIWA
+# Copyright (C) 2008-2018 NIWA & British Crown (Met Office) & Contributors.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,11 +15,18 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+"""Parsec - library for parsing nested Cylc's INI-style configuration."""
+
+import logging
 
 
 class ParsecError(Exception):
-    def __init__(self, msg):
-        self.msg = msg
+    def __init__(self, msg=''):
+        Exception.__init__(self, msg)
+        self.msg = self.args[0]
 
     def __str__(self):
         return str(self.msg)
+
+
+LOG = logging.getLogger('cylc')  # Acceptable?
